function status = Animate2(q,theta,flag,varargin)

persistent TARGET_FIGURE TARGET_AXIS 

status = 0;                             % Assume stop button wasn't pushed.
colorscheme = lines(7);
tracescheme = lines(10);

sqcolor = colorscheme(1,:);
surfacecolor = colorscheme(4,:);
tracecolor = colorscheme(7,:);

delay = nan;
surface = [];
offset = [];
phase = 1;
makemovie = 0;
plot_title = '';
plot_trace = 0;

for i=1:2:length(varargin)
    key = varargin{i};
    value = varargin{i+1};
    switch (key)
        case 'ground_surface'
            surface = value;
        case 'offset'
            offset = value;
        case 'delay'
            delay = value;
        case 'phase'
            phase = value;
        case 'movie'
            makemovie = 1;
        case 'title'
            plot_title = value;
        case 'trace'
            plot_trace = value;
    end
end


if nargin < 3 || isempty(flag)
    
    if (isempty(TARGET_FIGURE) || isempty(TARGET_AXIS))
        
        error('MATLAB:Animate2:NotCalledWithInit', ...
              'Animate2 has not been initialized. Use syntax Animate2(q,theta,''init'').');
        
    elseif (ishandle(TARGET_FIGURE)) % && ishandle(TARGET_AXIS))  % figure still open   

%         try 
            
            ud = get(TARGET_FIGURE,'UserData');
            figure(TARGET_FIGURE)
            if ~isnan(delay)
                ud.delay = delay;
            end
            if ~isempty(offset)
                ud.offset = offset;
            end
            if ~isempty(surface)
                ud.surface = surface;
            end

            if ~isempty(ud.surface)
                set (ud.SurfaceModel, 'xdata', ud.surface(:,1), 'ydata', ud.surface(:,2));
            end



            set(TARGET_FIGURE,'UserData',ud);  
            
            ud.theta = [ud.theta ; theta ];
            ud.sqdata = [ud.sqdata; q ];
            framecount = size(ud.sqdata, 1);
            
            if ud.plot_trace ~= 0
                subplot(ud.sp1)
                trace_line = line(0,0,'color',tracecolor,'erase','none','linewidth',1);
                hold on
            end   
            
            subplot(ud.sp(phase));
            hold on;
            newline = plot(0,0,'EraseMode','none');

            hold off;
            ud.colorindex = mod(ud.colorindex, length(colorscheme)) + 1;
            set(newline, 'color', colorscheme(ud.colorindex,:));

            ud.tmpx=[]; ud.tmpy=[];
            for i = ud.findex:framecount
                        
                [x,y] = qtox(ud.sqdata(i,:), ud.offset);
                
                co = [ud.offset ; x.' y.'];
                set(ud.SQModel,'xdata',co(:,1),'ydata',co(:,2));
                titlestring = [ud.plot_title ': Step ' num2str(i) '/' num2str(framecount)];
                set(ud.SQTitle,'String',titlestring);
                
                if (ud.plot_trace~=0)
                    ud.tmpx = [ud.tmpx; co(end,1)];ud.tmpy = [ud.tmpy; co(end,2)];
                    set(trace_line,'xdata',ud.tmpx,'ydata',ud.tmpy);
                end
                if (ud.plot_trace > 1 && mod(i,ud.plot_trace)==1)
                    ud.cidex=mod(ud.cidex,length(tracescheme))+1;
                    subplot(ud.sp1)
                    shadow = line(0,0,'color',tracescheme(ud.colorindex,:),'erase','none','linestyle','--','linewidth',1);
                    set(shadow,'xdata',co(:,1),'ydata',co(:,2));
                end
                set(newline,'xdata', ud.theta(ud.findex:i,1), 'ydata', ud.theta(ud.findex:i,2));
                pause(ud.delay);
                
                if ud.makemovie==1
                    F = getframe(TARGET_FIGURE);
                    ud.mov = addframe(ud.mov,F);
                end
                
                drawnow;
            end
            ud.findex = i+1;

            set(TARGET_FIGURE,'UserData',ud);               
            
%         catch
%             err = lasterror();
%             error('MATLAB:Animate2:ErrorUpdatingWindow',...
%                   'Error updating the Animate2 window. solution data may have been corrupted. %s',...
%                   err.message);
%         end                                          
    end
  
else
    
    switch(flag)
      case 'init'                           % odephas2(tspan,y0,'init')

        f = figure(gcf);
        
        TARGET_FIGURE = f;
        TARGET_AXIS = gca;

        ud.findex = 1;
        ud.sqdata = [];
        ud.theta = [];
        
        ud.zm = zoom;
%         set(ud.zm,'enable','on');        
        
        ud.sp1 = subplot(211);
        ylim([0 0.3]);
        xlim([-0.3 1.2]);
        orient landscape;
        xlabel('x (m)', 'interpreter', 'latex');
        ylabel('y (m)', 'interpreter', 'latex');

        ud.SQTitle = title('Motion Trajectories', 'interpreter', 'latex') ;
        ud.SQModel = line(0,0,'color',sqcolor,'erase','xor','linewidth',1, 'Marker','.');
        hold on;
        ud.SurfaceModel = line(0,0,'color',surfacecolor,'erase','xor','linewidth',1);
        hold off;

%         axis([-1 1 -1 1]);
%         axis image
        % xor for update behaviour
%         set (ud.SQModel,'Erase','xor');
%         set (ud.SQTitle,'Erase','xor');

        ud.sp(1) = subplot(223);
        ud.PhaseTitle = title('Phase plot 1', 'interpreter', 'latex');
        xlabel('$\theta$', 'interpreter', 'latex');
        ylabel('$\dot{\theta}$', 'interpreter', 'latex');
%         axis image fill;
        
        ud.sp(2) = subplot(224);
        ud.PhaseTitle = title('Phase plot 2', 'interpreter', 'latex');
        xlabel('$\theta$', 'interpreter', 'latex');
        ylabel('$\dot{\theta}$', 'interpreter', 'latex');       
%         axis image fill;

        ud.colorindex = 2;
        ud.cidex = 1;
        ud.plot_trace = plot_trace;
        if isnan(delay)
            delay = 0;
        end
        if isempty(offset)
            offset = [0 0];
        end
        ud.plot_title = plot_title;
        ud.delay = delay;
        ud.surface = surface;
        ud.offset = offset;
        ud.makemovie = 0;        
        if makemovie==1
            %### MOVIE GENERATION
            ud.makemovie = 1;

            ck=clock;
            ckyr = int2str(ck(1));
            ckhr=int2str(ck(4)+100);  ckday=int2str(ck(3)+100);
            ckmin=int2str(ck(5)+100); cksec=int2str(ck(6)+100);
            filename = sprintf('animation_%s_%s_%s.avi', ckhr(2:3),ckmin(2:3),cksec(2:3));
            ud.mov = avifile(filename);
        end
        % The STOP button.
        
%         h = findobj(f,'Tag','stop');
%         if isempty(h)
%             ud.stop = 0;
%             pos = get(0,'DefaultUicontrolPosition');
%             pos(1) = pos(1) - 15;
%             pos(2) = pos(2) - 15;            
%             uicontrol( ...
%                 'Style','push', ...
%                 'String','Stop', ...
%                 'Position',pos, ...
%                 'Callback',@StopButtonCallback, ...
%                 'Tag','stop');
%         else
%             set(h,'Visible','on');            % make sure it's visible
%             if ishold
%                 oud = get(f,'UserData');
%                 ud.stop = oud.stop;             % don't change old ud.stop status
%             else
%                 ud.stop = 0;
%             end
%         end
        set(f,'UserData',ud);
    
      case 'done'
        ud = get(TARGET_FIGURE,'UserData');          
        if ud.makemovie==1
            ud.mov = close(ud.mov);
        end
        TARGET_FIGURE = [];    
        TARGET_AXIS = [];     
    end
end

end


% --------------------------------------------------------------------------
% Sub-function
%

function StopButtonCallback(src,eventdata)
    ud = get(gcbf,'UserData'); 
    ud.stop = ~ud.stop;
    set(gcbf,'UserData',ud);
end  % StopButtonCallback

% --------------------------------------------------------------------------
