function score = RunCLSystem
    clear
    close all
    clc
    
    warning off
    
    global VERBOSE, VERBOSE = false;
    global THETA_0
    global PQ
    global step, step = 100;
    global R
    global Period
    global DESIRED_THETA
    
    global Rc, Rc = diag([1 1 1 1]*0.1);
    global Qc, Qc = diag([7 ones(1,4)*1 ones(1,4)*1]); 
    
%###############################
% TRAJECTORY PARAMETERS
%###############################

    %coming close
    thetaPC = 2.86;
    phiPC = [0.3, 2.7, 0.9, 0.9];
    
    %reaching far
    thetaPF = 2.9;
    phiPF = [0.6, 2.1, 0.4, 0.4;];
    
%###############################
% GENERAL DEFINITIONS
%###############################

    score = 0;
    refine = 4;
    options = [];
    
%###############################
% VARABLES FOR LOG
%###############################
    
    theta1 = [];
    theta2 = [];
    theta3 = [];
    theta4 = [];
    
    state1 = [];
    state2 = [];
    state3 = [];
    state4 = [];
    
    theta = [];    
    state = [];
    time = [];

    
    dIlog = [];
    feedback = [];
    Ylog = [];
    timetable = [];
%###############################
% PRELIMINARIES
%###############################

    Animate2([],[], 'init','title', 'Closed loop', 'ground_surface',[-10 0; 10 0], 'delay', 0.0, 'trace', 10);
    
%###############################
% FIRST PART OF FIRST STEP
%###############################

    %starting with legs in "close" position
    initial_configuration = [2.89471731132989, 0.529938155971379, 2.24384433771315, 0.570888280497162, 0.570888280497162];

    theta0 = initial_configuration(1);
    phi0 = initial_configuration(2:5);

    thetaP = thetaPC;
    phiP = phiPC;

    k = zeros(4,1);
    
    for i=1:size(k,1)
        k(i) = (phi0(i)-phiP(i))/(theta0-thetaP);
     end
    
    PQ(1,:) = [ k(1)  phiP(1)-k(1)*(thetaP)];
    PQ(2,:) = [ k(2)  phiP(2)-k(2)*(thetaP)];
    PQ(3,:) = [ k(3)  phiP(3)-k(3)*(thetaP)];
    PQ(4,:) = [ k(4)  phiP(4)-k(4)*(thetaP)];
 
    THETA_0 = theta0;    
    [R, X, Y] = SolveRiccatiEquation;

    options = odeset('Events',@SwitchEvent,'Refine',refine);
    [t1,y1,te1,ye1,ie1] = ode113(@DynamicsEquations, linspace(0, Period, step),...
        [ initial_configuration zeros(1,5) ], options);
    
    PP1 = PQ;

    state1 = y1(:,1:5);
    theta1 = [ y1(:,1) y1(:,6)];
    
    
    state = [ state; state1 ];
    theta = [ theta; theta1 ];     
    time = [ time ; t1 ];
    
    Animate2(state1, theta1, '');
    
%###############################
% SECOND PART OF FIRST STEP
%###############################
if true
    
    theta0 = ye1(end,1); 
    phi0 = ye1(end, 2:5);

    phiP = phiPF; 
    thetaP = thetaPF;
    k = zeros(4,1);
    
    for i=1:size(k,1)
        k(i) = (phi0(i)-phiP(i))/(theta0-thetaP);
     end    
    
    PQ(1,:) = [ k(1)  phiP(1)-k(1)*(thetaP)];
    PQ(2,:) = [ k(2)  phiP(2)-k(2)*(thetaP)];
    PQ(3,:) = [ k(3)  phiP(3)-k(3)*(thetaP)];
    PQ(4,:) = [ k(4)  phiP(4)-k(4)*(thetaP)];  

    THETA_0 = theta0;
    [R, X, Y] = SolveRiccatiEquation;
    
    options = odeset('Events',@SwitchEvent,'Refine',refine);
    [t2,y2,te2,ye2,ie2] = ode113(@DynamicsEquations, linspace(0, Period, step),...
        ye1(end,:), options);
    
    PP2=PQ;
    
    state2 = y2(:,1:5);
    theta2 = [ y2(:,1) y2(:,6) ];
        
    state = [ state; state2 ];
    theta = [ theta; theta2 ];
    time = [ time; time(end)+t2 ];
    
    Animate2(state2, theta2, '');
    
end    
%###############################
% SECOND STEP
%###############################



%###############################
% FIRST PART OF TRAJECTORY
%###############################

if (true)
    %extended configuration, reflected
    initial_configuration = [pi-y2(end,1), -y2(end,2:5)];
    
    theta0 = initial_configuration(1);
    phi0 = initial_configuration(2:5);
    
    thetaP = pi-thetaPF;
    phiP = -phiPF;

    k = zeros(4,1);
    for i=1:size(k,1)
        k(i) = (phi0(i)-phiP(i))/(theta0-thetaP);
     end
    
    PQ(1,:) = [ k(1)  phiP(1)-k(1)*(thetaP)];
    PQ(2,:) = [ k(2)  phiP(2)-k(2)*(thetaP)];
    PQ(3,:) = [ k(3)  phiP(3)-k(3)*(thetaP)];
    PQ(4,:) = [ k(4)  phiP(4)-k(4)*(thetaP)];
    
    THETA_0 = theta0;
    [R, X, Y] = SolveRiccatiEquation;
    
    options = odeset('Events',@SwitchEvent,'Refine',refine);
    [t3,y3,te3,ye3,ie3] = ode113(@DynamicsEquations, linspace(0, Period, step),...
        [ initial_configuration zeros(1,5) ], options);

    PP3 = PQ;
    
    state3 = y3(:,1:5);
    theta3 = [ y3(:,1) y3(:,6) ];
        
    state = [ state; state3 ];
    theta = [ theta; theta3 ];
    time = [ time; time(end)+t3 ];
    
    [xt,yt] = qtox(y2(end,1:5));
    Animate2(state3, theta3, '','phase',2, 'offset', [0.9 0]);%[xt(end) 0]);
end
%###############################
% SECOND PART OF TRAJECTORY
%###############################
if (true)
    theta0 = ye3(end,1);
    phi0 = ye3(end,2:5);

    thetaP = pi-thetaPC;
    phiP = -phiPC;
    
    k = zeros(4,1);
    for i=1:size(k,1)
        k(i) = (phi0(i)-phiP(i))/(theta0-thetaP);
     end    
    
    PQ(1,:) = [ k(1)  phiP(1)-k(1)*(thetaP)];
    PQ(2,:) = [ k(2)  phiP(2)-k(2)*(thetaP)];
    PQ(3,:) = [ k(3)  phiP(3)-k(3)*(thetaP)];
    PQ(4,:) = [ k(4)  phiP(4)-k(4)*(thetaP)];   
 
    THETA_0 = theta0;
    [R, X, Y] = SolveRiccatiEquation;
 
    options = odeset('Events',@SwitchEvent,'Refine',refine);
    [t4,y4,te4,ye4,ie4] = ode113(@DynamicsEquations, linspace(0, Period, step),...
        ye3(end,:), options);
    
    PP4=PQ;
    state4 = y4(:,1:5);
    theta4 = [ y4(:,1) y4(:,6) ];
        
    state = [ state; state4 ];
    theta = [ theta; theta4 ];
    time = [ time; time(end)+t4 ];
    
    Animate2(state4, theta4, '', 'phase',2, 'offset', [0.9 0]);%, [xt(end) 0]);
end   
%###############################
% PLOTTING RESULTS
%###############################
%%
    if (true)
        figure
        Animate2([],[],'done'); %end animation and save movie
        figure(64)
        plot(dIlog), title('dI', 'interpreter', 'latex')
        figure(65)
        plot(feedback), title('Feedback', 'interpreter', 'latex')
        figure(14)
        plot(state), title('States', 'interpreter', 'latex')
        figure(67)
        plot(Ylog), title('Y-values', 'interpreter', 'latex')
%         figure(68)
%         plot(real(EIGENVALUES(2,:)).')
    end

    %exit
    score=0;
    return


    
%###############################
% SYSTEM DYNAMICS
%###############################

    function [dx] = DynamicsEquations(t, x)
        
        %Feedback and theta interpolation
        real_idx = step*(mod(t, Period)/Period)+1;
        floor_idx = floor(real_idx);
        ceil_idx = mod(floor_idx, step)+1;
        idx_offset = real_idx-floor_idx;
        
        %interpolated desired theta and feedback matrix
        theta_ipl = (DESIRED_THETA(ceil_idx,:)-DESIRED_THETA(floor_idx,:))*idx_offset...
            + DESIRED_THETA(floor_idx,:);
        R_ipl = (R(:,:,ceil_idx)-R(:,:,floor_idx))*idx_offset ...
            + R(:,:,floor_idx);
        
        sprintf('time: %g/%g, floor: %d, ceil: %d \n', t, Period, floor_idx, ceil_idx)
        
        %Computing 
        Y = xtoy(x);
        I = integral(x);
        [ v, dI ] = TLController(theta_ipl, [I; Y], R_ipl);
        u = FeedbackTransform(x, v);
        dx = FiveDofDynamics(x, u);
        
        % journaling
        timetable = [timetable; t];
        Ylog = [Ylog ; Y.'];
        dIlog = [dIlog; dI];
        feedback = [feedback; u.'];
    end


%###############################
% EVENT FUNCTIONS
%###############################

    function [value,isterminal,direction] = SwitchEvent(t,y)
       %terminating when dtheta == 0
       value = y(6);
       isterminal = 1;
       direction = 0;
    end

    function [value,isterminal,direction] = ImpactEvent(t,y)
       % Terminating the integration routine the moment the front leg
       % touches ground.
       [xpf,ypf] = qtox(y(1:5,1).');
       value = ypf(5);
%        value = y(2);
       isterminal = 1;
%        direction = -1;
       direction = -1;
    end

%###############################
% HELPER FUNCTIONS
%###############################




end