function score = RunLoopSystem
    clear
    close all
    clc
    
    warning off
    
    global VERBOSE, VERBOSE = false;
    global THETA_0
    global PQ
    global step, step = 100;
    global R
    global Period
    global DESIRED_THETA
    
    global Rc, Rc = diag([1 1 1 1]*0.1);
    global Qc, Qc = diag([7 ones(1,4)*1 ones(1,4)*1]); 
    
%###############################
% TRAJECTORY PARAMETERS
%###############################

    %coming close
    thetaPC = 2.86;
    phiPC = [0.3, 2.7, 0.9, 0.9];
    
    %reaching far
    thetaPF = 2.9;
    phiPF = [0.6, 2.1, 0.4, 0.4;];
    
%###############################
% GENERAL DEFINITIONS
%###############################

    score = 0;
    refine = 4;
    options = [];
    number_of_periods = 10;
%###############################
% VARABLES FOR LOG
%###############################
    
    theta1 = [];
    theta2 = [];
    theta3 = [];
    theta4 = [];
    
    state1 = [];
    state2 = [];
    state3 = [];
    state4 = [];
    
    theta = [];    
    state = [];
    time = [];
    
    dIlog = [];
    feedback = [];
    Ylog = [];
    timetable = [];
%###############################
% PRELIMINARIES
%###############################
    figure
    Animate2([],[], 'init','title', 'Closed loop', 'ground_surface',[-10 0; 10 0], 'delay', 0.0, 'trace', 10);
    
%###############################
% FIRST PART OF FIRST STEP
%###############################

    %starting with legs in "close" position
    initial_configuration = [2.89471731132989, 0.529938155971379, 2.24384433771315, 0.570888280497162, 0.570888280497162];

    theta0 = initial_configuration(1);
    phi0 = initial_configuration(2:5);

    thetaP = thetaPC;
    phiP = phiPC;

    k = zeros(4,1);
    
    for i=1:size(k,1)
        k(i) = (phi0(i)-phiP(i))/(theta0-thetaP);
     end
    
    PQ(1,:) = [ k(1)  phiP(1)-k(1)*(thetaP)];
    PQ(2,:) = [ k(2)  phiP(2)-k(2)*(thetaP)];
    PQ(3,:) = [ k(3)  phiP(3)-k(3)*(thetaP)];
    PQ(4,:) = [ k(4)  phiP(4)-k(4)*(thetaP)];
 
    THETA_0 = theta0;    
    [R, X, Y] = SolveRiccatiEquation;
       
    [t1,y1] = ode113(@DynamicsEquations, linspace(0, Period*number_of_periods, step*number_of_periods),...
         [ initial_configuration + [0 1 0 0 0] zeros(1,5)], options);   


    state1 = y1(:,1:5);
    theta1 = [ y1(:,1) y1(:,6)];
   
    state = [ state; state1 ];
    theta = [ theta; theta1 ];     
    time = [ time ; t1 ];
    
    Animate2(state1, theta1, '');
    
%###############################
% PLOTTING RESULTS
%###############################
%%
    if (true)
        figure
        Animate2([],[],'done', 'interpreter', 'latex'); %end animation and save movie
        
        figure(64)
        plot(dIlog), title('dI', 'interpreter', 'latex')
        
        figure(65)
        plot(feedback), title('Torque', 'interpreter', 'latex')

        figure(14)
        subplot(121)
        plot(time, state), title('States', 'interpreter', 'latex')
        subplot(122)
        plot(theta(:,1), theta(:,2)), title('Phase plot', 'interpreter', 'latex');
        
        figure(67)
        subplot(131)
        plot(timetable, Ylog), title('Error coordinates', 'interpreter', 'latex')
        subplot(132)
        plot(timetable, Ylog), title('Error coordinates, zoom', 'interpreter', 'latex')
        subplot(133)
        plot(timetable, Ylog(:,1)), title('Error coordinates, general integral', 'interpreter', 'latex')

    end

    %exit
    score=0;
    return


    
%###############################
% SYSTEM DYNAMICS
%###############################

    function [dx] = DynamicsEquations(t, x)
        
        %Feedback and theta interpolation
        real_idx = step*(mod(t, Period)/Period)+1;
        floor_idx = floor(real_idx);
        ceil_idx = mod(floor_idx, step)+1;
        idx_offset = real_idx-floor_idx;
        
        %interpolated desired theta and feedback matrix
        theta_ipl = (DESIRED_THETA(ceil_idx,:)-DESIRED_THETA(floor_idx,:))*idx_offset...
            + DESIRED_THETA(floor_idx,:);
        R_ipl = (R(:,:,ceil_idx)-R(:,:,floor_idx))*idx_offset ...
            + R(:,:,floor_idx);
        
        sprintf('time: %g/%g, floor: %d, ceil: %d \n', t, Period, floor_idx, ceil_idx)

        %Computing 
        Y = xtoy(x);
        I = integral(x);
        [ v, dI ] = TLController(theta_ipl, [I; Y], R_ipl);
        u = FeedbackTransform(x, v);
        dx = FiveDofDynamics(x, u);
        
        % journaling
        timetable = [timetable; t];
        Ylog = [Ylog ; [I; Y].'];
        dIlog = [dIlog; dI.'];
        feedback = [feedback; u.'];
    end


%###############################
% EVENT FUNCTIONS
%###############################

    function [value,isterminal,direction] = SwitchEvent(t,y)
       %terminating when dtheta == 0
       value = y(6);
       isterminal = 1;
       direction = 0;
    end

    function [value,isterminal,direction] = ImpactEvent(t,y)
       % Terminating the integration routine the moment the front leg
       % touches ground.
       [xpf,ypf] = qtox(y(1:5,1).');
       value = ypf(5);
%        value = y(2);
       isterminal = 1;
%        direction = -1;
       direction = -1;
    end

%###############################
% HELPER FUNCTIONS
%###############################




end