function [ b ] = step_1_findcoeffs(Posits, Speeds)
% function [ thetacoeffs ] = step_1_findcoeffs(Posits, Speeds)
% Script for finding a suitable, monotonically increasing theta(q)
% This script utilizes a genetic algorithm to find coefficients
% that ensure a monotonical theta(q). Found by finding a sign definite 
% theta'(q)
% Lode 13.02.09

clear FIRST_RUN
%%Loading Nantes data.
% load statedata.mat

%% Flight interval
lspd = Speeds;
lpos = Posits;

%% Extracting discrete values from position and velocity tables
% Joint values (1->4) at intervals of 1/50th of the entire section

spd = lspd(1:round(length(lspd)/50):length(lspd),:);
pos = lpos(1:round(length(lpos)/50):length(lpos),:);


%% Genetic algorithm
% theta = a1*qi + a2*sin(qi) + a3*sin(qi/2) + ...
% theta' = a1*dqi + a2*cos(qi)*dqi + a3*cos(qi/2)*dqi + ...

% Select alphas from within this interval
bounds = [-1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1;
           1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1];
% Run ga() 
[b, sc] = ga('step_1_coeff_fun', bounds,[20,-1,0.12,100,3000,500,1e-4,50,0], pos, spd)

% Extract found coefficients
alpha1 = b(1:5).';
alpha2 = b(6:10).';
alpha3 = b(11:15).';
alpha4 = b(16:20).';
%% Plot results

% Preparing the trig. components
for i=1:max(size(lpos))
    for j=1:min(size(lpos))
        %for theta'
        cosar1(i,j) = cos(lpos(i,j))*lspd(i,j);
        cosar2(i,j) = 1/2*cos(lpos(i,j)/2)*lspd(i,j);
        cosar3(i,j) = 1/4*cos(lpos(i,j)/4)*lspd(i,j);
        % for theta
        sinar1(i,j) = sin(lpos(i,j));
        sinar2(i,j) = sin(lpos(i,j)/2);
        sinar3(i,j) = sin(lpos(i,j)/4);
    end
end
vhc_pos = lpos*alpha1 + sinar1*alpha2 + sinar2*alpha3 + sinar3*alpha4;
vhc_vel = lspd*alpha1 + cosar1*alpha2 + cosar2*alpha3 + cosar3*alpha4;

figure
orient tall
subplot(121),plot(vhc_pos), title('$$\theta$$', 'interpreter','latex'), zoom on;
subplot(122),plot(vhc_vel, 'r'), title('$$\dot\theta$$', 'interpreter','latex'),  zoom on;

figure
orient tall
 
subplot(421),plot(vhc_pos), title('theta'),  zoom on;
subplot(422),plot(vhc_vel, 'r'), title('theta`'), zoom on;

subplot(423),plot(vhc_pos, lpos(:,1)), title('q1 vs theta'),  zoom on;
subplot(424),plot(vhc_pos, lpos(:,2)), title('q2 vs theta'),  zoom on;
subplot(425),plot(vhc_pos, lpos(:,3)), title('q3 vs theta'),  zoom on;
subplot(426),plot(vhc_pos, lpos(:,4)), title('q4 vs theta'),  zoom on;
subplot(427),plot(vhc_pos, lpos(:,5)), title('q5 vs theta'), zoom on;

end
