function [ X, fh ] = step_2_findphis_fun(Posits, Speeds, Accels, state, b, N)
%% This script finds a polynomial which fits the system
% [q dq ddq ]' = [theta(q) dtheta(q,dq) ddtheta(q, dq, ddq)]'*[b1 ... b4]'
%
%
% q(i): the positions
% dq(i): the velocities
% ddq(i): the accelerations
%

% 
% theta = [q(i)   sin(q(i))       sin(q(i)/2)     sin(q(i)/4) ];
% 
% dtheta = [dq(i)     cos(q(i))*dq(i)     1/2*cos(q(i)/2)*dq(i) 1/4*cos(q(i)/4)*dq(i) ];
% 
% ddtheta = [ qqd(i)                                              cos(q(i))*ddq(i)-sin(q(i))*dq(i)^2  ... 
%             1/2*cos(q(i)/2)*ddq(i)-1/4*sin(q(i)/2)*dq(i)^2      1/4*cos(q(i)/4)*ddq(i)-1/16*sin(q(i)/4)*dq(i)^2  ];
% for i=1:50
%     tstart1(i,:)=Posits(1,:);
%     tend1(i,:)=Posits(length(Posits),:);
%     tstart2(i,:)=Speeds(1,:);
%     tend2(i,:)=Speeds(length(Speeds),:);
%     tstart3(i,:)=Accels(1,:);
%     tend3(i,:)=Accels(length(Accels),:);
% end
% 
% lpos = [tstart1; Posits; tend1;];
% lspd = [tstart2; Speeds; tend2;];
% lacc = [tstart3; Accels; tend3;];
lpos = [Posits;];
lspd = [Speeds;];
lacc = [Accels;];

alpha1 = b(1:5).';
alpha2 = b(6:10).';
alpha3 = b(11:15).';
alpha4 = b(16:20).';

% Preparing the trig. components
for i=1:max(size(lpos))
    for j=1:min(size(lpos))
        % for theta
        theta1(i,j) = sin(lpos(i,j));
        theta2(i,j) = sin(lpos(i,j)/2);
        theta3(i,j) = sin(lpos(i,j)/4);
        %for theta'
        dtheta1(i,j) = cos(lpos(i,j))*lspd(i,j);
        dtheta2(i,j) = 1/2*cos(lpos(i,j)/2)*lspd(i,j);
        dtheta3(i,j) = 1/4*cos(lpos(i,j)/4)*lspd(i,j);
        %for theta''
        ddtheta1(i,j) = cos(lpos(i,j))*lacc(i,j) - sin(lpos(i,j))*lspd(i,j)^2;
        ddtheta2(i,j) = 1/2*cos(lpos(i,j)/2)*lacc(i,j) - 1/4*sin(lpos(i,j)/2)*lspd(i,j)^2;
        ddtheta3(i,j) = 1/4*cos(lpos(i,j)/4)*lacc(i,j) - 1/16*sin(lpos(i,j)/4)*lspd(i,j)^2;        
    end
end

theta = lpos*alpha1 + theta1*alpha2 + theta2*alpha3 + theta3*alpha4;
dtheta = lspd*alpha1 + dtheta1*alpha2 + dtheta2*alpha3 + dtheta3*alpha4;
ddtheta = lacc*alpha1 + ddtheta1*alpha2 + ddtheta2*alpha3 + ddtheta3*alpha4;


% 

% N = 20;
A = zeros(length(lpos), N+1);
A(:,1)=1;
dA = zeros(length(lspd), N+1);
ddA = zeros(length(lacc), N+1);

for i=1:1:max(size(theta))
    for (j=1:1:(N))
        A(i,j+1) = theta(i)^(j);
        dA(i,j+1) = theta(i)^(j-1)*(j)*dtheta(i);
        ddA(i,j+1) = j*theta(i)^(j-1)*ddtheta(i) + j*(j-1)*theta(i)^(j-2)*dtheta(i)^2;
    end
end

bigA = [A;dA;ddA];
bigY = [lpos(:,state); lspd(:,state); lacc(:,state)];

%normalizing
% for i=1:1:max(size(bigA))
%    bigA(i,:)=bigA(i,:)./norm(bigA(1,:));
%    bigY(i)=bigY(i)./norm(bigA(1,:));
% end

X = bigA\bigY;

fh = figure;
title(state)
subplot(311), plot(theta, A*X,'g',theta,lpos(:,state),'-.'), title(['$$q_{',num2str(state),'}$$'], 'interpreter','latex', 'fontsize', 12)
subplot(312), plot(theta, dA*X,'g',theta,lspd(:,state),'-.'), title(['$$\dot{q}_{',num2str(state),'}$$'], 'interpreter','latex', 'fontsize', 12)
subplot(313), plot(theta, ddA*X,'g',theta,lacc(:,state),'-.'), title(['$$\ddot{q}_{',num2str(state),'}$$'], 'interpreter','latex', 'fontsize', 12)

end