function score = RunSystem(k, runplot)

    pivot = 2.9;
    k0 =    2.5;
    k1 =    -2;
    k2 =    -0.5;
    k3 =    -0.5;
    PP(1,:) = [0    k0  0.60-k0*(pivot)];
    PP(2,:) = [0    k1  2.10-k1*(pivot)];
    PP(3,:) = [0    k2  0.40-k2*(pivot)];
    PP(4,:) = [0    k3  0.40-k3*(pivot)];
    refine = 4;
    options = odeset('Events',@SwitchEvent,'Refine',refine);
    
    [t1,y1,te1,ye1,ie1] = ode113(@DynamicsEquations, [0 10], [2.7 0], options);
    PP1 = PP;
    
    theta = [y1];
    state = [y1(:,1) ThetaToPhi(y1(:,1), PP1)];
    time = [t1];
    [xp,yp] = qtox(state);

    
    
    %Generate constraints for the last part of the trajectory
    pivot = ye1(1);

    
    % Linear constraints
    k0 = k(1);
    k1 = k(2);
    k2 = k(3);
    k3 = k(4);
    stat = ThetaToPhi(ye1(1), PP);
    PP(1,:) = [0 k0     stat(1)-k0*(pivot)  ];
    PP(2,:) = [0 k1     stat(2)-k1*(pivot)  ];
    PP(3,:) = [0 k2     stat(3)-k2*(pivot)  ];
    PP(4,:) = [0 k3     stat(4)-k3*(pivot)  ];

    % Cubic constraints
%     k0 = k(1);
%     k1 = k(2);
%     k2 = k(3);
%     k3 = k(4);
%     l0 = k(5);
%     l1 = k(6);
%     l2 = k(7);
%     l3 = k(8);
% 
%     stat = ThetaToPhi(ye1(1), PP);
%     PP(1,:) = [k0   -2*k0*(pivot)+l0  stat(1)+k0*pivot^2-l0*pivot];
%     PP(2,:) = [k1   -2*k1*(pivot)+l1  stat(2)+k1*pivot^2-l1*pivot];
%     PP(3,:) = [k2   -2*k2*(pivot)+l2  stat(3)+k2*pivot^2-l2*pivot];
%     PP(4,:) = [k3   -2*k3*(pivot)+l3  stat(4)+k3*pivot^2-l3*pivot];

   
    options = odeset('Events',@ImpactEvent,'Refine',refine);
    [t2,y2,te2,ye2,ie2] = ode113(@DynamicsEquations, [te1(1) te1(1)+2], ye1, options);
    PP2=PP;
    theta = [theta; y2];
    state = [state;
             y2(:,1) ThetaToPhi(y2(:,1), PP2)];
    time = [time;t2];
    
    %% Transform generalized- to Cartesian coordinates along the path
    [xp,yp] = qtox(state);
    
    
    %% Plot all useful diagrams etc.
    if (nargin > 1)
        figure(12)
        Animate(state, theta, 0.07)
        
        figure(13)
        plot(time, state)

        figure(14)
        subplot(331), plot(y1(:,1),CalcGamma(y1(:,1),PP1)), hold on, plot(y2(:,1),CalcGamma(y2(:,1), PP2),'r'),title('gamma vs theta'), zoom on
        subplot(332), plot(t1, CalcGamma(y1(:,1), PP1)), hold on, plot(t2, CalcGamma(y2(:,1), PP2),'r' ),title('gamma'),zoom on
        subplot(333), plot(time, state), title('states vs. time'),zoom on
        subplot(334), plot(y1(:,1),y1(:,2)),hold on, plot(y2(:,1),y2(:,2), 'r'), title('theta vs. dtheta'),zoom on
        subplot(335), plot(t1, y1),hold on, plot(t2,y2,'r'), title('theta vs. time'), zoom on
        subplot(336), plot(t1,CalcAlpha(y1(:,1).', PP1)), hold on, plot(t2,CalcAlpha(y2(:,1).', PP2), 'r'), title('alpha'), zoom on
        subplot(337), plot(xp(:,5)), title('x pos of tip vs. time'), zoom on
        subplot(338), plot(state(:,1),yp(:,5)), title('y pos of tip vs. theta'), zoom on
        subplot(339), plot(yp(:,5)), title('y pos of tip vs. time'), zoom on
    end

    %% confirm that the end configuration is valid - sanity test
    % all joints in upper half plane, front leg in touch with ground
    
    len = length(yp);
    score = 0;   
    if ( (min(min(yp)) < 0) )
%         disp '(min(min(yp)) < 0)'; 
%         score = score + exp(min(min(yp)));
          score = score + 1000;
    end
    if ( (min(min(state)) < 0) )
%         disp '(min(min(state)) < 0)';
%         score = score + exp(min(min(state)));
          score = score + 1000;
    end
    if ( (max(max(state)) > pi) )
%         disp '(max(max(state)) > pi)';
%         score = score + exp(max(max(state)));
          score = score + 1000;
    end
    
    %% Calculate optimality-score:

    %% performance criterium: speed of theta
%     score = score + (y2(2))^2;
    %% performance criterium: step length
    step_length = 0.7;
    score = score + exp((step_length-xp(len,5))) ;
    %% performance criterium: speed of end point on impact
%     good_speed = 0.3;
%     score = score + (good_speed-(yp(len,5)-yp(len-10,5))/(time(len)-time(len-10)) )^2;
    %% performance criterium: speed of front leg on impact
    
    return

    %%
    function [dx] = DynamicsEquations(t, x)
        dx = ConstrainedDynamics(x, PP);
    end

    function [value,isterminal,direction] = ImpactEvent(t,y)
       % Terminating the integration routine the moment the front leg
       % touches ground.
       [xpf,ypf] = qtox([y(1) ThetaToPhi(y(1),PP)]);
       value = ypf(5);
       isterminal = 1;
       direction = -1;
    end

    function [value,isterminal,direction] = SwitchEvent(t,y)
       % Terminating the integration at theta = 2.8 going in decreasing direction
%        value = y(1)-3;
       value = y(2);
       isterminal = 1;
       direction = -1;
    end

end