function score = StepRunSystem
clear
    

%###############################
% TRAJECTORY PARAMETERS
%###############################

    %coming close
    thetaPC = 2.86;
    phiPC = [0.3, 2.7, 0.9, 0.9];
    
    %reaching far
    thetaPF = 2.9;
    phiPF = [0.6, 2.1, 0.4, 0.4;];
    

%###############################
% GENERAL DEFINITIONS
%###############################

    score = 0;
    refine = 4;
    slope_angle = 3*pi/180;
    step_height = 0.1;
    

%###############################
% VARABLES FOR LOG
%###############################
    
    q1 = [];
    q2 = [];
    q3 = [];
    q4 = [];
    
    theta1 = [];
    theta2 = [];
    theta3 = [];
    theta4 = [];
    
    torque1 = [];
    torque2 = [];
    torque3 = [];
    torque4 = [];
    
    %first step
    state1 = [];
    time1 = [];
    theta_changes1 = [];
    
    %second step
    state2 = [];
    time2 = [];
    theta_changes2 = [];

    
    
%###############################
% PRELIMINARIES
%###############################

    Animate2([],[], 'init','ground_surface',[-10 0 ; 10 0], 'delay', 0.0, 'trace',5,'title','Reduced Dynamics');
    
%###############################
% FIRST STEP
%###############################

%starting in "close" position
initial_configuration = [2.89471731132989, 0.529938155971379, 2.24384433771315, 0.570888280497162, 0.570888280497162];

%###############################
% FIRST PART OF TRAJECTORY
%###############################
if (true)

    theta0 = initial_configuration(1);
    phi0 = initial_configuration(2:5);

    thetaP = thetaPC;
    phiP = phiPC;

    k = zeros(4,1);
    
    for i=1:size(k,1)
        k(i) = (phi0(i)-phiP(i))/(theta0-thetaP);
     end
    
    PP(1,:) = [0    k(1)  phiP(1)-k(1)*(thetaP)];
    PP(2,:) = [0    k(2)  phiP(2)-k(2)*(thetaP)];
    PP(3,:) = [0    k(3)  phiP(3)-k(3)*(thetaP)];
    PP(4,:) = [0    k(4)  phiP(4)-k(4)*(thetaP)];

    options = odeset('Events',@SwitchEvent,'Refine',refine);
    [t1,y1,te1,ye1,ie1] = ode45(@DynamicsEquations, [0 2], [theta0 0], options);
    
    PP1 = PP;
    q1 = [y1(:,1) ThetaToPhi(y1(:,1), PP1)];

    theta1 = y1;
    state1 = [state1 ; y1(:,1) ThetaToPhi(y1(:,1), PP1)];
    time1 = [time1 ; t1];

    torque1 = calc_accel(theta1, PP1);
    
    Animate2(q1, theta1, '');
end

%###############################
% SECOND PART OF TRAJECTORY
%###############################
if (true)

    theta0 = lastr(ye1,1); 
    phi0 = ThetaToPhi(theta0, PP1);

    phiP = phiPF; 
    thetaP = thetaPF;
    k = zeros(4,1);
    
    for i=1:size(k,1)
        k(i) = (phi0(i)-phiP(i))/(theta0-thetaP);
     end    
    
    PP(1,:) = [0    k(1)  phiP(1)-k(1)*(thetaP)];
    PP(2,:) = [0    k(2)  phiP(2)-k(2)*(thetaP)];
    PP(3,:) = [0    k(3)  phiP(3)-k(3)*(thetaP)];
    PP(4,:) = [0    k(4)  phiP(4)-k(4)*(thetaP)];    
    
    options = odeset('Events',@ImpactEvent,'Refine',refine);
    [t2,y2,te2,ye2,ie2] = ode45(@DynamicsEquations, [lastr(te1) lastr(te1)+2], [theta0 0] , options);
    
    PP2=PP;
    q2 = [y2(:,1) ThetaToPhi(y2(:,1), PP2)];    
    theta2 = y2;
    state1 = [state1; y2(:,1) ThetaToPhi(y2(:,1), PP2)];
    time1 = [time1;t2];
    Animate2(q2, theta2, '');
    torque2 = calc_accel(theta2, PP2);    
end
  
%###############################
% SECOND STEP
%###############################

%extended configuration, reflected
initial_configuration = [pi-lastr(ye2,1), -ThetaToPhi(lastr(ye2,1), PP2)];

%###############################
% FIRST PART OF TRAJECTORY
%###############################
if (true)

    theta0 = initial_configuration(1);
    phi0 = initial_configuration(2:5);
    
    thetaP = pi-thetaPF;
    phiP = -phiPF;

%     phi0 = [0.3,2.645,0.1,0.1;]; % initial position  
    k = zeros(4,1);
    
    for i=1:size(k,1)
        k(i) = (phi0(i)-phiP(i))/(theta0-thetaP);
     end
    
    PP(1,:) = [0    k(1)  phiP(1)-k(1)*(thetaP)];
    PP(2,:) = [0    k(2)  phiP(2)-k(2)*(thetaP)];
    PP(3,:) = [0    k(3)  phiP(3)-k(3)*(thetaP)];
    PP(4,:) = [0    k(4)  phiP(4)-k(4)*(thetaP)];

    options = odeset('Events',@SwitchEvent,'Refine',refine);
    [t3,y3,te3,ye3,ie3] = ode45(@DynamicsEquations, [lastr(te2) lastr(te2)+2], [theta0 0], options);
    
    PP3 = PP;
    q3 = [y3(:,1) ThetaToPhi(y3(:,1), PP3)];
    theta3 = y3;
    state2 = [state2 ; y3(:,1) ThetaToPhi(y3(:,1), PP3)];
    time2 = [time2 ; t3];
    Animate2(q3, theta3, '','offset', [0.9 0], 'phase', 2);    
    torque3 = calc_accel(theta3, PP3);    
end

%###############################
% SECOND PART OF TRAJECTORY
%###############################
if (true)

    theta0 = lastr(ye3,1);
    phi0 = ThetaToPhi(theta0, PP3);

    thetaP = pi-thetaPC;
    phiP = -phiPC;
    
    k = zeros(4,1);
    
    for i=1:size(k,1)
        k(i) = (phi0(i)-phiP(i))/(theta0-thetaP);
     end    
    
    PP(1,:) = [0    k(1)  phiP(1)-k(1)*(thetaP)];
    PP(2,:) = [0    k(2)  phiP(2)-k(2)*(thetaP)];
    PP(3,:) = [0    k(3)  phiP(3)-k(3)*(thetaP)];
    PP(4,:) = [0    k(4)  phiP(4)-k(4)*(thetaP)];    
    
    options = odeset('Events',@ImpactEvent,'Refine',refine);
    [t4,y4,te4,ye4,ie4] = ode45(@DynamicsEquations, [lastr(te3) lastr(te3)+2], [theta0 0] , options); % ye1 [te1(1) te1(1)+1]
    
    PP4=PP;
    q4 = [y4(:,1) ThetaToPhi(y4(:,1), PP4)];
    theta4 = y4;
    state2 = [state2; y4(:,1) ThetaToPhi(y4(:,1), PP4)];
    time2 = [time2;t4];
    Animate2(q4, theta4, '','offset', [0.9 0], 'phase', 2);
    torque4 = calc_accel(theta4, PP4);    
end

%###############################
% ANALYSIS
%###############################


        


%###############################
% PLOTTING RESULTS
%###############################
%%
    if (true)
        state = [state1; state2];
        time = [time1; time2];
        [xp,yp] = qtox(state);
        
        %## END ANIMATION, SAVE MOVIE
        Animate2([],[],'done');
        figure
        theta = [theta1; theta2; theta3; theta4 ];
        torque = [torque1; torque2 ;torque3 ;torque4];
        normal_force = zeros(length(theta),1);
        horizontal_force = zeros(length(theta),1);

        [l, s, m, Ia, Iy, N, g] = RobDef();
        for i=1:(length(theta))
            normal_force(i) = torque(i)/l(1)*sin(theta(i,1)-pi/2) + m(1)*g;
            horizontal_force(i) = torque(i)/l(1)*cos(theta(i,1)-pi/2);
        end

        figure
        plot(normal_force);title('Normal force');
        hold on
        plot(horizontal_force, 'r');
%         plot(torque,'g');
        legend('Normal', 'Tangent', 'Torque');
        figure(14)
        plot(time, state);
        
        if (true)
            figure(15)
            subplot(331)
                plot(y1(:,1),CalcGamma(y1(:,1),PP1)), hold on
                plot(y2(:,1),CalcGamma(y2(:,1),PP2)), hold on
                plot(y3(:,1),CalcGamma(y3(:,1),PP3)), hold on
                plot(y4(:,1),CalcGamma(y4(:,1),PP4))
                title('gamma vs theta')
                zoom on
            subplot(332)
                plot(t1, CalcGamma(y1(:,1), PP1)), hold on
                plot(t2, CalcGamma(y2(:,1), PP2)), hold on
                plot(t3, CalcGamma(y3(:,1), PP3)), hold on
                plot(t4, CalcGamma(y4(:,1), PP4),'r' )
                title('gamma')
                zoom on
            subplot(333)
                plot(time, state), title('states vs. time')
                zoom on
            subplot(334)
                plot(y1(:,1),y1(:,2)),hold on
                plot(y2(:,1),y2(:,2), 'r')
                title('theta vs. dtheta step 1')
                zoom on
            subplot(335)
                plot(y3(:,1),y3(:,2)),hold on
                plot(y4(:,1),y4(:,2), 'r')
                title('theta vs. dtheta step 2')
                zoom on
            subplot(336)
                plot(t1, y1),hold on
                plot(t2, y2, 'r'),hold on
                plot(t3, y3, 'c'),hold on
                plot(t4, y4, 'm')
                title('theta vs. time')
                zoom on
            subplot(337)
                plot(t1,CalcAlpha(y1(:,1).', PP1)), hold on
                plot(t2,CalcAlpha(y2(:,1).', PP2), 'r')
                title('alpha')
                zoom on
            subplot(338)
                plot(abs(xp(:,5))), title('distance feet vs. time')
                zoom on
            subplot(339)
                plot(yp(:,5)), title('height of swing leg vs. time')
                zoom on
        end
    end

    %% confirm that the end configuration is valid - sanity test
    % all joints in upper half plane, front leg in touch with ground
 
    if ( (min(min(yp)) < 0) )
%         disp '(min(min(yp)) < 0)'; 
        score = -10E3;
        return
    end
%     if ( (yp(length(yp),5) > 0) )
% %         disp '(yp(length(yp),5) > 0)';
%         score = -10E3;
%         return
%     end
    if ( (min(min(state)) < 0) )
%         disp '(min(min(state)) < 0)';
        score = -10E3;
        return
    end
    if ( (max(max(state)) > pi) )
%         disp '(max(max(state)) > pi)';
        score = -10E3;
        return
    end
    
    return


    
%###############################
% SYSTEM DYNAMICS
%###############################

    function [dx] = DynamicsEquations(t, x)
        dx = ConstrainedDynamics(x, PP);
    end


%###############################
% EVENT FUNCTIONS
%###############################

    function [value,isterminal,direction] = SwitchEvent(t,y)
       % Terminating the integration at theta = 2.8 going in decreasing direction
%        value = y(1)-3;
       value = y(2);
       isterminal = 1;
       direction = 0;
    end

    function [value,isterminal,direction] = ImpactEvent(t,y)
       % Terminating the integration routine the moment the front leg
       % touches ground.
       [xpf,ypf] = qtox([y(1) ThetaToPhi(y(1),PP)]);
       value = ypf(5);
%        value = y(2);
       isterminal = 1;
%        direction = -1;
       direction = -1;
    end

%###############################
% HELPER FUNCTIONS
%###############################
    function [t] = calc_accel(theta, PP)
        t = zeros(length(theta),1);
        for i=1:length(t)
            tmp = ConstrainedDynamics(theta(i,:), PP);
            t(i,:) = tmp(2);
        end
    end
    function [e] = lastr(x,i)
        e = x(size(x,1),:);
        if (nargin>1)
            e = e(i);
        end
    end

    function [e] = lastc(x,i)
        e = x(:,size(x,2));
        if (nargin>1)
            e = e(i);
        end
    end



end